@@ -16,6 +16,11 @@
 from neutron import context as nctx
 import neutron.db.api as db
 from neutron.db import db_base_plugin_v2
+from neutron.db import securitygroups_db as sec_db
+from neutron.db import segments_db
+from neutron.plugins.common import constants as p_const
+from neutron.plugins.ml2 import driver_api
+from neutron.plugins.ml2 import models as ml2_models
 
 from networking_arista.common import db as db_models
 
@@ -29,8 +34,11 @@ def remember_tenant(tenant_id):
     """
     session = db.get_session()
     with session.begin():
-        tenant = db_models.AristaProvisionedTenants(tenant_id=tenant_id)
-        session.add(tenant)
+        tenant = (session.query(db_models.AristaProvisionedTenants).
+                  filter_by(tenant_id=tenant_id).first())
+        if not tenant:
+            tenant = db_models.AristaProvisionedTenants(tenant_id=tenant_id)
+            session.add(tenant)
 
 
 def forget_tenant(tenant_id):
@@ -79,8 +87,19 @@ def remember_vm(vm_id, host_id, port_id, network_id, tenant_id):
         session.add(vm)
 
 
-def update_vm_host(vm_id, host_id, port_id, network_id, tenant_id):
-    """Updates the VM host id in the database.
+def forget_all_ports_for_network(net_id):
+    """Removes all ports for a given network fron repository.
+
+    :param net_id: globally unique network ID
+    """
+    session = db.get_session()
+    with session.begin():
+        (session.query(db_models.AristaProvisionedVms).
+         filter_by(network_id=net_id).delete())
+
+
+def update_port(vm_id, host_id, port_id, network_id, tenant_id):
+    """Updates the port details in the database.
 
     :param vm_id: globally unique identifier for VM instance
     :param host_id: ID of the new host where the VM is placed
@@ -90,58 +109,66 @@ def update_vm_host(vm_id, host_id, port_id, network_id, tenant_id):
     """
     session = db.get_session()
     with session.begin():
-        vm = session.query(db_models.AristaProvisionedVms).filter_by(
-            vm_id=vm_id, port_id=port_id, tenant_id=tenant_id,
-            network_id=network_id).first()
-        if vm:
+        port = session.query(db_models.AristaProvisionedVms).filter_by(
+            port_id=port_id).first()
+        if port:
             # Update the VM's host id
-            vm.host_id = host_id
+            port.host_id = host_id
+            port.vm_id = vm_id
+            port.network_id = network_id
+            port.tenant_id = tenant_id
 
 
-def forget_vm(vm_id, host_id, port_id, network_id, tenant_id):
-    """Removes all relevant information about a VM from repository.
+def forget_port(port_id, host_id):
+    """Deletes the port from the database
 
-    :param vm_id: globally unique identifier for VM instance
-    :param host_id: ID of the host where the VM is placed
     :param port_id: globally unique port ID that connects VM to network
-    :param network_id: globally unique neutron network identifier
-    :param tenant_id: globally unique neutron tenant identifier
+    :param host_id: host to which the port is bound to
     """
     session = db.get_session()
     with session.begin():
-        (session.query(db_models.AristaProvisionedVms).
-         filter_by(vm_id=vm_id, host_id=host_id,
-                   port_id=port_id, tenant_id=tenant_id,
-                   network_id=network_id).delete())
+        session.query(db_models.AristaProvisionedVms).filter_by(
+            port_id=port_id,
+            host_id=host_id).delete()
 
 
-def remember_network(tenant_id, network_id, segmentation_id):
+def remember_network_segment(tenant_id,
+                             network_id, segmentation_id, segment_id):
     """Stores all relevant information about a Network in repository.
 
     :param tenant_id: globally unique neutron tenant identifier
     :param network_id: globally unique neutron network identifier
-    :param segmentation_id: VLAN ID that is assigned to the network
+    :param segmentation_id: segmentation id that is assigned to the network
+    :param segment_id: globally unique neutron network segment identifier
     """
     session = db.get_session()
     with session.begin():
         net = db_models.AristaProvisionedNets(
             tenant_id=tenant_id,
+            id=segment_id,
             network_id=network_id,
             segmentation_id=segmentation_id)
         session.add(net)
 
 
-def forget_network(tenant_id, network_id):
+def forget_network_segment(tenant_id, network_id, segment_id=None):
     """Deletes all relevant information about a Network from repository.
 
     :param tenant_id: globally unique neutron tenant identifier
     :param network_id: globally unique neutron network identifier
+    :param segment_id: globally unique neutron network segment identifier
     """
+    filters = {
+        'tenant_id': tenant_id,
+        'network_id': network_id
+    }
+    if segment_id:
+        filters['id'] = segment_id
+
     session = db.get_session()
     with session.begin():
         (session.query(db_models.AristaProvisionedNets).
-         filter_by(tenant_id=tenant_id, network_id=network_id).
-         delete())
+         filter_by(**filters).delete())
 
 
 def get_segmentation_id(tenant_id, network_id):
@@ -180,25 +207,48 @@ def is_vm_provisioned(vm_id, host_id, port_id,
         return num_vm > 0
 
 
-def is_network_provisioned(tenant_id, network_id, seg_id=None):
+def is_port_provisioned(port_id, host_id=None):
+    """Checks if a port is already known to EOS
+
+    :returns: True, if yes; False otherwise.
+    :param port_id: globally unique port ID that connects VM to network
+    :param host_id: host to which the port is bound to
+    """
+
+    filters = {
+        'port_id': port_id
+    }
+    if host_id:
+        filters['host_id'] = host_id
+
+    session = db.get_session()
+    with session.begin():
+        num_ports = (session.query(db_models.AristaProvisionedVms).
+                     filter_by(**filters).count())
+        return num_ports > 0
+
+
+def is_network_provisioned(tenant_id, network_id, segmentation_id=None,
+                           segment_id=None):
     """Checks if a networks is already known to EOS
 
     :returns: True, if yes; False otherwise.
     :param tenant_id: globally unique neutron tenant identifier
     :param network_id: globally unique neutron network identifier
-    :param seg_id: Optionally matches the segmentation ID (VLAN)
+    :param segment_id: globally unique neutron network segment identifier
     """
     session = db.get_session()
     with session.begin():
-        if not seg_id:
-            num_nets = (session.query(db_models.AristaProvisionedNets).
-                        filter_by(tenant_id=tenant_id,
-                                  network_id=network_id).count())
-        else:
-            num_nets = (session.query(db_models.AristaProvisionedNets).
-                        filter_by(tenant_id=tenant_id,
-                                  network_id=network_id,
-                                  segmentation_id=seg_id).count())
+        filters = {'tenant_id': tenant_id,
+                   'network_id': network_id}
+        if segmentation_id:
+            filters['segmentation_id'] = segmentation_id
+        if segment_id:
+            filters['id'] = segment_id
+
+        num_nets = (session.query(db_models.AristaProvisionedNets).
+                    filter_by(**filters).count())
+
         return num_nets > 0
 
 
@@ -277,20 +327,50 @@ def get_vms(tenant_id):
         # hack for pep8 E711: comparison to None should be
         # 'if cond is not None'
         none = None
-        all_vms = (session.query(model).
-                   filter(model.tenant_id == tenant_id,
-                          model.host_id != none,
-                          model.vm_id != none,
-                          model.network_id != none,
-                          model.port_id != none))
-        res = dict(
-            (vm.vm_id, vm.eos_vm_representation())
-            for vm in all_vms
-        )
-        return res
+        all_ports = (session.query(model).
+                     filter(model.tenant_id == tenant_id,
+                            model.host_id != none,
+                            model.vm_id != none,
+                            model.network_id != none,
+                            model.port_id != none))
+        ports = {}
+        for port in all_ports:
+            if port.port_id not in ports:
+                ports[port.port_id] = port.eos_port_representation()
+            else:
+                ports[port.port_id]['hosts'].append(port.host_id)
+
+        vm_dict = dict()
+
+        def eos_vm_representation(port):
+            return {u'vmId': port['deviceId'],
+                    u'baremetal_instance': False,
+                    u'ports': [port]}
+
+        for port in ports.values():
+            deviceId = port['deviceId']
+            if deviceId in vm_dict:
+                vm_dict[deviceId]['ports'].append(port)
+            else:
+                vm_dict[deviceId] = eos_vm_representation(port)
+        return vm_dict
 
 
-def get_ports(tenant_id):
+def are_ports_attached_to_network(net_id):
+    """Returns all records associated with network in EOS-compatible format.
+
+    :param net_id: globally unique network ID
+    """
+    session = db.get_session()
+    with session.begin():
+        model = db_models.AristaProvisionedVms
+        num_ports = (session.query(model).
+                     filter(model.network_id == net_id).count())
+
+        return num_ports > 0
+
+
+def get_ports(tenant_id=None):
     """Returns all ports of VMs in EOS-compatible format.
 
     :param tenant_id: globally unique neutron tenant identifier
@@ -301,17 +381,27 @@ def get_ports(tenant_id):
         # hack for pep8 E711: comparison to None should be
         # 'if cond is not None'
         none = None
-        all_ports = (session.query(model).
-                     filter(model.tenant_id == tenant_id,
-                            model.host_id != none,
-                            model.vm_id != none,
-                            model.network_id != none,
-                            model.port_id != none))
-        res = dict(
-            (port.port_id, port.eos_port_representation())
-            for port in all_ports
-        )
-        return res
+        if tenant_id:
+            all_ports = (session.query(model).
+                         filter(model.tenant_id == tenant_id,
+                                model.host_id != none,
+                                model.vm_id != none,
+                                model.network_id != none,
+                                model.port_id != none))
+        else:
+            all_ports = (session.query(model).
+                         filter(model.tenant_id != none,
+                                model.host_id != none,
+                                model.vm_id != none,
+                                model.network_id != none,
+                                model.port_id != none))
+    ports = {}
+    for port in all_ports:
+        if port.port_id not in ports:
+            ports[port.port_id] = port.eos_port_representation()
+        ports[port.port_id]['hosts'].append(port.host_id)
+
+    return ports
 
 
 def get_tenants():
@@ -327,7 +417,34 @@ def get_tenants():
         return res
 
 
-class NeutronNets(db_base_plugin_v2.NeutronDbPluginV2):
+def _make_bm_port_dict(record):
+    """Make a dict from the BM profile DB record."""
+    return {'port_id': record.port_id,
+            'host_id': record.host,
+            'vnic_type': record.vnic_type,
+            'profile': record.profile}
+
+
+def get_all_baremetal_ports():
+    """Returns a list of all ports that belong to baremetal hosts."""
+    session = db.get_session()
+    with session.begin():
+        querry = session.query(ml2_models.PortBinding)
+        bm_ports = querry.filter_by(vnic_type='baremetal').all()
+
+        return {bm_port.port_id: _make_bm_port_dict(bm_port)
+                for bm_port in bm_ports}
+
+
+def get_port_binding_level(session, filters):
+    """Returns entries from PortBindingLevel based on the specified filters."""
+    with session.begin():
+        return (session.query(ml2_models.PortBindingLevel).
+                filter_by(**filters).all())
+
+
+class NeutronNets(db_base_plugin_v2.NeutronDbPluginV2,
+                  sec_db.SecurityGroupDbMixin):
     """Access to Neutron DB.
 
     Provides access to the Neutron Data bases for all provisioned
@@ -363,11 +480,39 @@ class NeutronNets(db_base_plugin_v2.NeutronDbPluginV2):
     def get_shared_network_owner_id(self, network_id):
         filters = {'id': [network_id]}
         nets = self.get_networks(self.admin_ctx, filters=filters) or []
-        if not nets:
+        segments = segments_db.get_network_segments(self.admin_ctx.session,
+                                                    network_id)
+        if not nets or not segments:
             return
-        if nets[0]['shared']:
+        if (nets[0]['shared'] and
+           segments[0][driver_api.NETWORK_TYPE] == p_const.TYPE_VLAN):
             return nets[0]['tenant_id']
 
+    def get_network_segments(self, network_id, dynamic=False, session=None):
+        db_session = session if session else self.admin_ctx.session
+        segments = segments_db.get_network_segments(db_session, network_id,
+                                                    filter_dynamic=dynamic)
+        if dynamic:
+            for segment in segments:
+                segments['is_dynamic'] = True
+        return segments
+
+    def get_all_network_segments(self, network_id, session=None):
+        segments = self.get_network_segments(network_id, session=session)
+        segments += self.get_network_segments(network_id, dynamic=True,
+                                              session=session)
+        return segments
+
+    def get_segment_by_id(self, session, segment_id):
+        return segments_db.get_segment_by_id(session,
+                                             segment_id)
+
+    def get_network_from_net_id(self, network_id, context=None):
+        filters = {'id': [network_id]}
+        ctxt = context if context else self.admin_ctx
+        return super(NeutronNets,
+                     self).get_networks(ctxt, filters=filters) or []
+
     def _get_network(self, tenant_id, network_id):
         filters = {'tenant_id': [tenant_id],
                    'id': [network_id]}
@@ -404,3 +549,30 @@ class NeutronNets(db_base_plugin_v2.NeutronDbPluginV2):
     def get_port(self, port_id):
         return super(NeutronNets,
                      self).get_port(self.admin_ctx, port_id) or {}
+
+    def get_all_security_gp_to_port_bindings(self):
+        return super(NeutronNets, self)._get_port_security_group_bindings(
+            self.admin_ctx) or []
+
+    def get_security_gp_to_port_bindings(self, sec_gp_id):
+        filters = {'security_group_id': [sec_gp_id]}
+        return super(NeutronNets, self)._get_port_security_group_bindings(
+            self.admin_ctx, filters=filters) or []
+
+    def get_security_group(self, sec_gp_id):
+        return super(NeutronNets,
+                     self).get_security_group(self.admin_ctx, sec_gp_id) or []
+
+    def get_security_groups(self):
+        sgs = super(NeutronNets,
+                    self).get_security_groups(self.admin_ctx) or []
+        sgs_all = {}
+        if sgs:
+            for s in sgs:
+                sgs_all[s['id']] = s
+        return sgs_all
+
+    def get_security_group_rule(self, sec_gpr_id):
+        return super(NeutronNets,
+                     self).get_security_group_rule(self.admin_ctx,
+                                                   sec_gpr_id) or []
