@@ -15,7 +15,7 @@
 
 
 import mock
-from oslo.config import cfg
+from oslo_config import cfg
 
 from neutron.tests import base
 
