@@ -14,9 +14,10 @@
 # limitations under the License.
 
 import mock
+from neutron_lib import constants as n_const
 
-from neutron.common import constants as n_const
 from neutron.extensions import portbindings
+from neutron.plugins.ml2 import driver_api as api
 from neutron.tests.unit import testlib_api
 
 from networking_arista.ml2 import mechanism_arista
@@ -36,6 +37,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.fake_rpc = mock.MagicMock()
         mechanism_arista.db_lib = self.fake_rpc
         self.drv = mechanism_arista.AristaDriver(self.fake_rpc)
+        self.drv.ndb = mock.MagicMock()
 
     def tearDown(self):
         super(AristaDriverTestCase, self).tearDown()
@@ -46,17 +48,21 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         network_id = 'net1-id'
         segmentation_id = 1001
 
+        self.drv.rpc.hpb_supported.return_value = True
         network_context = self._get_network_context(tenant_id,
                                                     network_id,
                                                     segmentation_id,
                                                     False)
         self.drv.create_network_precommit(network_context)
+        segment_id = network_context.network_segments[0]['id']
 
         expected_calls = [
+            mock.call.hpb_supported(),
             mock.call.remember_tenant(tenant_id),
-            mock.call.remember_network(tenant_id,
-                                       network_id,
-                                       segmentation_id)
+            mock.call.remember_network_segment(tenant_id,
+                                               network_id,
+                                               segmentation_id,
+                                               segment_id)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -73,12 +79,15 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                     False)
         network_context.current['tenant_id'] = ''
         self.drv.create_network_precommit(network_context)
+        segment_id = network_context.network_segments[0]['id']
 
         expected_calls += [
+            mock.call.hpb_supported(),
             mock.call.remember_tenant(INTERNAL_TENANT_ID),
-            mock.call.remember_network(INTERNAL_TENANT_ID,
-                                       network_id,
-                                       segmentation_id)
+            mock.call.remember_network_segment(INTERNAL_TENANT_ID,
+                                               network_id,
+                                               segmentation_id,
+                                               segment_id)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -97,7 +106,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         segments = network_context.network_segments
         net_dict = {
             'network_id': network['id'],
-            'segmentation_id': segments[0]['segmentation_id'],
+            'segments': segments,
             'network_name': network['name'],
             'shared': network['shared']}
 
@@ -126,7 +135,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         segments = network_context.network_segments
         net_dict = {
             'network_id': network['id'],
-            'segmentation_id': segments[0]['segmentation_id'],
+            'segments': segments,
             'network_name': network['name'],
             'shared': network['shared']}
 
@@ -158,7 +167,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         expected_calls = [
             mock.call.is_network_provisioned(tenant_id, network_id),
             mock.call.are_ports_attached_to_network(network_id),
-            mock.call.forget_network(tenant_id, network_id),
+            mock.call.forget_network_segment(tenant_id, network_id),
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -184,7 +193,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         expected_calls += [
             mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id),
             mock.call.are_ports_attached_to_network(network_id),
-            mock.call.forget_network(INTERNAL_TENANT_ID, network_id),
+            mock.call.forget_network_segment(INTERNAL_TENANT_ID, network_id),
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -222,6 +231,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         network_id = 'net1-id'
         segmentation_id = 1001
 
+        self.drv.rpc.hpb_supported.return_value = True
         network_context = self._get_network_context(tenant_id,
                                                     network_id,
                                                     segmentation_id,
@@ -231,7 +241,9 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         mechanism_arista.db_lib.num_vms_provisioned.return_value = 0
         self.drv.delete_network_postcommit(network_context)
         expected_calls = [
-            mock.call.delete_network(tenant_id, network_id),
+            mock.call.hpb_supported(),
+            mock.call.delete_network(tenant_id, network_id,
+                                     network_context.network_segments),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
             mock.call.forget_tenant(tenant_id),
@@ -254,7 +266,9 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         self.drv.delete_network_postcommit(network_context)
         expected_calls += [
-            mock.call.delete_network(INTERNAL_TENANT_ID, network_id),
+            mock.call.hpb_supported(),
+            mock.call.delete_network(INTERNAL_TENANT_ID, network_id,
+                                     network_context.network_segments),
             mock.call.num_nets_provisioned(INTERNAL_TENANT_ID),
             mock.call.num_vms_provisioned(INTERNAL_TENANT_ID),
             mock.call.forget_tenant(INTERNAL_TENANT_ID),
@@ -263,7 +277,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
-    def test_create_port_precommit(self):
+    def _test_create_port_precommit(self):
         tenant_id = 'ten-1'
         network_id = 'net1-id'
         segmentation_id = 1001
@@ -279,6 +293,10 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                               vm_id,
                                               network_context)
         mechanism_arista.db_lib.is_network_provisioned.return_value = True
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+
         host_id = port_context.current['binding:host_id']
         port_id = port_context.current['id']
         self.drv.create_port_precommit(port_context)
@@ -309,6 +327,10 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                               network_context)
         port_context.current['tenant_id'] = ''
         mechanism_arista.db_lib.is_network_provisioned.return_value = True
+
+        network = {'tenant_id': ''}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+
         host_id = port_context.current['binding:host_id']
         port_id = port_context.current['id']
         self.drv.create_port_precommit(port_context)
@@ -322,7 +344,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         ]
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
-    def test_create_port_postcommit(self):
+    def _test_create_port_postcommit(self):
         tenant_id = 'ten-1'
         network_id = 'net1-id'
         segmentation_id = 1001
@@ -340,21 +362,28 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         mechanism_arista.db_lib.is_network_provisioned.return_value = True
         mechanism_arista.db_lib.get_shared_network_owner_id.return_value = 1
 
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+
         port = port_context.current
         device_id = port['device_id']
         device_owner = port['device_owner']
         host_id = port['binding:host_id']
         port_id = port['id']
         port_name = port['name']
+        profile = port['binding:profile']
 
         self.drv.create_port_postcommit(port_context)
 
         expected_calls = [
+            mock.call.NeutronNets(),
+            mock.call.__nonzero__(),
             mock.call.is_port_provisioned(port_id),
             mock.call.is_network_provisioned(tenant_id, network_id, None),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, tenant_id,
-                                             port_name, device_owner)
+                                             port_name, device_owner, None,
+                                             [], None, switch_bindings=profile)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -379,6 +408,9 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         mechanism_arista.db_lib.is_network_provisioned.return_value = True
         mechanism_arista.db_lib.get_shared_network_owner_id.return_value = 1
 
+        network = {'tenant_id': ''}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+
         port = port_context.current
         device_id = port['device_id']
         device_owner = port['device_owner']
@@ -394,7 +426,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                              None),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, INTERNAL_TENANT_ID,
-                                             port_name, device_owner)
+                                             port_name, device_owner, None,
+                                             [], None, switch_bindings=profile)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -478,19 +511,38 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         device_id = port['device_id']
         host_id = port['binding:host_id']
         port_id = port['id']
+        vnic_type = port['binding:vnic_type']
+        profile = port['binding:profile']
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+        physnet = dict(physnet='default')
+        self.fake_rpc.get_physical_network.return_value = physnet
+        self.drv.rpc.hpb_supported.return_value = True
 
         self.drv.delete_port_postcommit(port_context)
 
         expected_calls = [
-            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.NeutronNets(),
+            mock.call.__nonzero__(),
+            mock.call.get_physical_network(host_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None,
+                                             None),
             mock.call.unplug_port_from_network(device_id, 'compute', host_id,
-                                               port_id, network_id, tenant_id),
+                                               port_id, network_id, tenant_id,
+                                               None, vnic_type,
+                                               switch_bindings=profile),
+            mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
             mock.call.forget_tenant(tenant_id),
             mock.call.delete_tenant(tenant_id),
+            mock.call.hpb_supported(),
         ]
-
+        for binding_level in port_context.binding_levels:
+            expected_calls.append(mock.call.is_network_provisioned(tenant_id,
+                                  network_id, None,
+                                  binding_level['bound_segment']['id']))
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
         # If there is no tenant id associated with the network, then the
@@ -514,27 +566,42 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         host_id = port['binding:host_id']
         port_id = port['id']
 
+        network = {'tenant_id': ''}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+        physnet = dict(physnet='default')
+        self.fake_rpc.get_physical_network.return_value = physnet
+
         self.drv.delete_port_postcommit(port_context)
 
         expected_calls += [
+            mock.call.get_physical_network(host_id),
             mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
-                                             None),
+                                             None, None),
             mock.call.unplug_port_from_network(device_id, 'compute', host_id,
                                                port_id, network_id,
-                                               INTERNAL_TENANT_ID),
+                                               INTERNAL_TENANT_ID, None,
+                                               vnic_type,
+                                               switch_bindings=profile),
+            mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(INTERNAL_TENANT_ID),
             mock.call.num_vms_provisioned(INTERNAL_TENANT_ID),
             mock.call.forget_tenant(INTERNAL_TENANT_ID),
             mock.call.delete_tenant(INTERNAL_TENANT_ID),
+            mock.call.hpb_supported(),
         ]
+        for binding_level in port_context.binding_levels:
+            expected_calls.append(mock.call.is_network_provisioned(
+                                  INTERNAL_TENANT_ID, network_id, None,
+                                  binding_level['bound_segment']['id']))
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
     def test_update_port_precommit(self):
 
         # Test the case where the port was not provisioned previsouly
+        # If port is not provisioned, we should bail out
         mechanism_arista.db_lib.is_port_provisioned.return_value = False
-        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+        mechanism_arista.db_lib.is_network_provisioned.return_value = False
 
         tenant_id = 'ten-1'
         network_id = 'net1-id'
@@ -553,10 +620,22 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         host_id = port_context.current['binding:host_id']
         port_context.original['binding:host_id'] = 'ubuntu0'
         port_id = port_context.current['id']
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+
+        # Make sure the port is not found
+        mechanism_arista.db_lib.is_port_provisioned.return_value = False
+
         self.drv.update_port_precommit(port_context)
+        segment_id = network_context.network_segments[-1]['id']
 
         expected_calls = [
-            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.NeutronNets(),
+            mock.call.__nonzero__(),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             segmentation_id,
+                                             segment_id),
             mock.call.is_port_provisioned(port_id, None),
             mock.call.remember_tenant(tenant_id),
             mock.call.remember_vm(vm_id, host_id, port_id,
@@ -565,7 +644,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
-        # Test the case where the port was not provisioned
+        # Test the case where the port was provisioned, but it was not on
+        # correct network. We should bail out in this case as well
         tenant_id = 'ten-2'
         network_id = 'net2-id'
         segmentation_id = 1002
@@ -583,14 +663,23 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         port_context.original['binding:host_id'] = 'ubuntu0'
         port_id = port_context.current['id']
 
-        # Force the check to return port found
+        # Force the check to return port found, but, network was not found
         mechanism_arista.db_lib.is_port_provisioned.return_value = True
-        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+        mechanism_arista.db_lib.is_network_provisioned.return_value = False
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
 
-        self.drv.update_port_precommit(port_context)
+        try:
+            self.drv.update_port_precommit(port_context)
+        except Exception:
+            # This shoud raise an exception as this is not permitted
+            # operation
+            pass
 
+        segment_id = network_context.network_segments[-1]['id']
         expected_calls += [
-            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             segmentation_id, segment_id),
             mock.call.is_port_provisioned(port_id, None),
             mock.call.update_port(vm_id, host_id, port_id,
                                   network_id, tenant_id)
@@ -614,23 +703,26 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                               vm_id,
                                               network_context)
         # Port does not contain a tenant
-        port_context.current['tenant_id'] = ''
+        port_context.current['tenant_id'] = None
         host_id = port_context.current['binding:host_id']
         port_context.original['binding:host_id'] = 'ubuntu0'
         port_id = port_context.current['id']
 
-        # Force the check to return port not found
-        mechanism_arista.db_lib.is_port_provisioned.return_value = False
+        # Force the check to return port and network were found
+        mechanism_arista.db_lib.is_port_provisioned.return_value = True
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+        network = {'tenant_id': None}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
 
         self.drv.update_port_precommit(port_context)
 
+        segment_id = network_context.network_segments[-1]['id']
         expected_calls += [
             mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
-                                             None),
+                                             segmentation_id, segment_id),
             mock.call.is_port_provisioned(port_id, None),
-            mock.call.remember_tenant(INTERNAL_TENANT_ID),
-            mock.call.remember_vm(vm_id, host_id, port_id,
-                                  network_id, INTERNAL_TENANT_ID)
+            mock.call.update_port(vm_id, host_id, port_id, network_id,
+                                  INTERNAL_TENANT_ID)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -648,7 +740,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.update_port_precommit(port_context)
         expected_calls += [
             mock.call.is_network_provisioned(tenant_id, network_id,
-                                             None),
+                                             segmentation_id, segment_id),
             mock.call.is_port_provisioned(port_id, host_id),
             mock.call.remember_tenant(tenant_id),
             mock.call.remember_vm(router_id, host_id, port_id,
@@ -661,7 +753,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.update_port_precommit(port_context)
         expected_calls += [
             mock.call.is_network_provisioned(tenant_id, network_id,
-                                             None),
+                                             segmentation_id, segment_id),
             mock.call.forget_port(port_id, 'ubuntu1'),
         ]
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -676,6 +768,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                     network_id,
                                                     segmentation_id,
                                                     False)
+        segments = network_context.network_segments
         port_context = self._get_port_context(tenant_id,
                                               network_id,
                                               vm_id,
@@ -684,10 +777,12 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         mechanism_arista.db_lib.is_port_provisioned.return_value = True
         mechanism_arista.db_lib.is_network_provisioned.return_value = True
         mechanism_arista.db_lib.get_shared_network_owner_id.return_value = 1
-        mechanism_arista.db_lib.get_segmentation_id.return_value = 1001
         mechanism_arista.db_lib.num_nets_provisioned.return_value = 1
         mechanism_arista.db_lib.num_vms_provisioned.return_value = 1
+        self.drv.ndb.get_all_network_segments.return_value = segments
 
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
         port = port_context.current
         device_id = port['device_id']
         device_owner = port['device_owner']
@@ -696,24 +791,39 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         port_context.original['binding:host_id'] = orig_host_id
         port_id = port['id']
         port_name = port['name']
+        vnic_type = port['binding:vnic_type']
+        profile = port['binding:profile']
+        network_name = network_context.current['name']
 
+        self.drv.rpc.hpb_supported.return_value = True
         self.drv.update_port_postcommit(port_context)
 
         expected_calls = [
             mock.call.NeutronNets(),
-            mock.call.get_segmentation_id(tenant_id, network_id),
+            mock.call.__nonzero__(),
             mock.call.is_port_provisioned(port_id, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
-                                             segmentation_id),
-            mock.call.is_network_provisioned(tenant_id, network_id, None),
+                                             segmentation_id, None),
+            mock.call.hpb_supported(),
+            mock.call.create_network_segments(tenant_id, network_id,
+                                              network_name,
+                                              segments),
+            mock.call.is_network_provisioned(tenant_id, network_id, None,
+                                             None),
             mock.call.unplug_port_from_network(device_id, 'compute',
                                                orig_host_id, port_id,
-                                               network_id, tenant_id),
+                                               network_id, tenant_id,
+                                               None, vnic_type,
+                                               switch_bindings=profile),
+            mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, tenant_id,
-                                             port_name, device_owner)
+                                             port_name, device_owner, None,
+                                             None, vnic_type,
+                                             segments=segments,
+                                             switch_bindings=profile)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -729,6 +839,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                     network_id,
                                                     segmentation_id,
                                                     False)
+        segments = network_context.network_segments
         port_context = self._get_port_context(tenant_id,
                                               network_id,
                                               vm_id,
@@ -738,9 +849,12 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         mechanism_arista.db_lib.is_port_provisioned.return_value = True
         mechanism_arista.db_lib.is_network_provisioned.return_value = True
         mechanism_arista.db_lib.get_shared_network_owner_id.return_value = 1
-        mechanism_arista.db_lib.get_segmentation_id.return_value = 1002
         mechanism_arista.db_lib.num_nets_provisioned.return_value = 1
         mechanism_arista.db_lib.num_vms_provisioned.return_value = 1
+        self.drv.ndb.get_all_network_segments.return_value = segments
+
+        network = {'tenant_id': ''}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
 
         port = port_context.current
         device_id = port['device_id']
@@ -750,25 +864,35 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         port_context.original['binding:host_id'] = orig_host_id
         port_id = port['id']
         port_name = port['name']
+        network_name = network_context.current['name']
 
         self.drv.update_port_postcommit(port_context)
 
         expected_calls += [
-            mock.call.get_segmentation_id(INTERNAL_TENANT_ID, network_id),
             mock.call.is_port_provisioned(port_id, None),
             mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
-                                             segmentation_id),
+                                             segmentation_id, None),
+            mock.call.hpb_supported(),
+            mock.call.create_network_segments(INTERNAL_TENANT_ID, network_id,
+                                              network_name,
+                                              segments),
             mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
-                                             None),
+                                             None, None),
             mock.call.unplug_port_from_network(device_id, 'compute',
                                                orig_host_id,
                                                port_id, network_id,
-                                               INTERNAL_TENANT_ID),
+                                               INTERNAL_TENANT_ID,
+                                               None, vnic_type,
+                                               switch_bindings=profile),
+            mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(INTERNAL_TENANT_ID),
             mock.call.num_vms_provisioned(INTERNAL_TENANT_ID),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, INTERNAL_TENANT_ID,
-                                             port_name, device_owner)
+                                             port_name, device_owner, None,
+                                             None, vnic_type,
+                                             segments=segments,
+                                             switch_bindings=profile)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -783,6 +907,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                     network_id,
                                                     segmentation_id,
                                                     False)
+        segments = network_context.network_segments
+        network_name = network_context.current['name']
         owner = n_const.DEVICE_OWNER_DVR_INTERFACE
         port_context = self._get_port_context(tenant_id,
                                               network_id,
@@ -793,9 +919,9 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         mechanism_arista.db_lib.is_port_provisioned.return_value = True
         mechanism_arista.db_lib.is_network_provisioned.return_value = True
         mechanism_arista.db_lib.get_shared_network_owner_id.return_value = 1
-        mechanism_arista.db_lib.get_segmentation_id.return_value = 1003
         mechanism_arista.db_lib.num_nets_provisioned.return_value = 1
         mechanism_arista.db_lib.num_vms_provisioned.return_value = 1
+        self.drv.ndb.get_all_network_segments.return_value = segments
 
         # New DVR port - context.original_host is not set and status is ACTIVE
         #                port should be plugged into the network
@@ -806,17 +932,25 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         orig_host_id = 'ubuntu1'
         port_id = port['id']
         port_name = port['name']
+        vnic_type = port['binding:vnic_type']
+        profile = port['binding:profile']
 
         self.drv.update_port_postcommit(port_context)
 
         expected_calls += [
-            mock.call.get_segmentation_id(tenant_id, network_id),
             mock.call.is_port_provisioned(port_id, port_context.host),
             mock.call.is_network_provisioned(tenant_id, network_id,
-                                             segmentation_id),
+                                             segmentation_id, None),
+            mock.call.hpb_supported(),
+            mock.call.create_network_segments(tenant_id, network_id,
+                                              network_name,
+                                              segments),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, tenant_id,
-                                             port_name, device_owner)
+                                             port_name, device_owner, None,
+                                             None, vnic_type,
+                                             segments=segments,
+                                             switch_bindings=profile)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -827,28 +961,37 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.update_port_postcommit(port_context)
 
         expected_calls += [
-            mock.call.get_segmentation_id(tenant_id, network_id),
             mock.call.is_port_provisioned(port_id, port_context.host),
             mock.call.is_network_provisioned(tenant_id, network_id,
-                                             segmentation_id),
+                                             segmentation_id, None),
+            mock.call.hpb_supported(),
+            mock.call.create_network_segments(tenant_id, network_id,
+                                              network_name,
+                                              segments),
             mock.call.is_network_provisioned(tenant_id, network_id,
-                                             None),
+                                             None, None),
             mock.call.unplug_port_from_network(device_id, owner,
                                                orig_host_id,
                                                port_id, network_id,
-                                               tenant_id),
+                                               tenant_id,
+                                               None, vnic_type,
+                                               switch_bindings=profile),
+            mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
-    def _get_network_context(self, tenant_id, net_id, seg_id, shared):
+    def _get_network_context(self, tenant_id, net_id,
+                             segmentation_id, shared):
         network = {'id': net_id,
                    'tenant_id': tenant_id,
                    'name': 'test-net',
                    'shared': shared}
-        network_segments = [{'segmentation_id': seg_id,
+        network_segments = [{'segmentation_id': segmentation_id,
+                             'physical_network': u'default',
+                             'id': 'segment-id-for-%s' % segmentation_id,
                              'network_type': 'vlan'}]
         return FakeNetworkContext(tenant_id, network, network_segments,
                                   network)
@@ -861,7 +1004,11 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                 'name': 'test-port',
                 'tenant_id': tenant_id,
                 'id': 101,
-                'network_id': net_id
+                'network_id': net_id,
+                'binding:vnic_type': None,
+                'binding:profile': [],
+                'security_groups': None,
+                'status': 'ACTIVE',
                 }
         orig_port = {'device_id': device_id,
                      'device_owner': device_owner,
@@ -869,9 +1016,20 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                      'name': 'test-port',
                      'tenant_id': tenant_id,
                      'id': 101,
-                     'network_id': net_id
+                     'network_id': net_id,
+                     'binding:vnic_type': None,
+                     'binding:profile': [],
+                     'security_groups': None,
+                     'status': 'ACTIVE',
                      }
-        return FakePortContext(port, dict(orig_port), network, status)
+        binding_levels = []
+        for level, segment in enumerate(network.network_segments):
+            binding_levels.append(FakePortBindingLevel(port['id'],
+                                                       level,
+                                                       'vendor-1',
+                                                       segment['id']))
+        return FakePortContext(port, dict(orig_port), network, status,
+                               binding_levels)
 
 
 class fake_keystone_info_class(object):
@@ -913,11 +1071,14 @@ class FakeNetworkContext(object):
 class FakePortContext(object):
     """To generate port context for testing purposes only."""
 
-    def __init__(self, port, original_port, network, status):
+    def __init__(self, port, original_port, network, status,
+                 binding_levels):
+        self._plugin_context = FakePluginContext('test')
         self._port = port
         self._original_port = original_port
         self._network_context = network
         self._status = status
+        self._binding_levels = binding_levels
 
     @property
     def current(self):
@@ -943,9 +1104,43 @@ class FakePortContext(object):
     def status(self):
         return self._status
 
+    @property
+    def original_status(self):
+        if self._original_port:
+            return self._original_port['status']
+
+    @property
+    def binding_levels(self):
+        if self._binding_levels:
+            return [{
+                api.BOUND_DRIVER: level.driver,
+                api.BOUND_SEGMENT: self._expand_segment(level.segment_id)
+            } for level in self._binding_levels]
+
+    @property
+    def bottom_bound_segment(self):
+        if self._binding_levels:
+            return self._expand_segment(self._binding_levels[-1].segment_id)
+
+    def _expand_segment(self, segment_id):
+        for segment in self._network_context.network_segments:
+            if segment[api.ID] == segment_id:
+                return segment
+
 
 class FakePluginContext(object):
     """Plugin context for testing purposes only."""
 
     def __init__(self, tenant_id):
         self.tenant_id = tenant_id
+        self.session = mock.MagicMock()
+
+
+class FakePortBindingLevel(object):
+    """Port binding object for testing purposes only."""
+
+    def __init__(self, port_id, level, driver, segment_id):
+        self.port_id = port_id
+        self.level = level
+        self.driver = driver
+        self.segment_id = segment_id
